package it.amattioli.applicate.browsing;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import it.amattioli.applicate.commands.CommandEvent;
import it.amattioli.dominate.Entity;
import it.amattioli.dominate.Repository;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

public class MultiBrowser<I extends Serializable, T extends Entity<I>> extends ListBrowserImpl<I, T> {
    private static final Logger logger = LoggerFactory.getLogger(MultiBrowser.class);
    private List<Integer> selectedIndexes = new ArrayList<Integer>();

    protected MultiBrowser() {

    }

    public MultiBrowser(Repository<I, T> repository) {
        super(repository);
    }

    @Override
	protected void invalidateContent() {
    	selectedIndexes.clear();
		super.invalidateContent();
	}

	public void select(final Integer index) {
		if (index == null) {
			selectedIndexes.clear();
			super.select((Integer)null);
		} else {
	    	if (selectedIndexes.contains(index)) {
	    		selectedIndexes.remove(index);
	    		if (selectedIndexes.isEmpty()) {
	    			super.select((Integer)null);
	    		} else {
	    			super.select(selectedIndexes.get(0));
	    		}
	    	} else {
	    		selectedIndexes.add(0, index);
	    		super.select(index);
	    	}
		}
    }
	
	@Override
	protected void refreshContent() {
		super.refreshContent();
		for (Iterator<Integer> iter = selectedIndexes.iterator(); iter.hasNext();) {
			Integer idx = iter.next();
			if (!isValidSelectionIndex(idx)) {
				iter.remove();
			}
		}
	}

	@Override
    public void commandDone(final CommandEvent source) {
        List<Integer> idxs = new ArrayList<Integer>(getSelectedIndexes().size());
		idxs.addAll(getSelectedIndexes());
        invalidateContent();
        for (Integer idx: idxs) {
	        if (isValidSelectionIndex(idx)) {
	        	select(idx);
	        }
        }
        fireCommandEvent(source);
    }

    public List<Integer> getSelectedIndexes() {
    	return Collections.unmodifiableList(selectedIndexes);
    }

    public List<T> getSelectedObjects() {
    	List<T> result = new ArrayList<T>();
    	for (Integer index : selectedIndexes) {
    		result.add(refreshObject(index));
    	}
    	return result;
    }
}
